Package: gss
Version: 2.2-10
Date: 2025-11-18
Title: General Smoothing Splines
Authors@R: c(person("Chong","Gu",role=c("aut","cre"),
                   email="chong@purdue.edu"))											
Author: Chong Gu [aut, cre]
Maintainer: Chong Gu <chong@purdue.edu>
Depends: R (>= 3.0.0), stats
Description: A comprehensive package for structural multivariate
        function estimation using smoothing splines.
License: GPL (>= 2)
Packaged: 2025-11-18 17:35:57 UTC; chong
NeedsCompilation: yes
Repository: CRAN
Date/Publication: 2025-11-19 06:10:18 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-26 10:30:29 UTC; unix
Archs: gss.so.dSYM
